/****************************************************************************
*    This file is part of CEDEC_GO.                                         *
*    Copyright (c) 2004-2011 Takenori Kubo, Youichiro Miyake, CESA          *
*                                                                           *
*    History:                                                               *
*    01/07/2011 Tweaked for CEDEC CHALLANGE 2011 by CESA                    *
*                                                                           *
*    CEDEC_GO is free software: you can redistribute it and/or modify       *
*    it under the terms of the GNU General Public License as published by   *
*    the Free Software Foundation, either version 3 of the License, or      *
*    any later version.                                                     *
*                                                                           *
*    CEDEC_GO is distributed in the hope that it will be useful,            *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of         *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
*    GNU General Public License for more details.                           *
*                                                                           *
*    You should have received a copy of the GNU General Public License      *
*    along with CEDEC_GO.  If not, see <http://www.gnu.org/licenses/>.      *
*****************************************************************************/

#ifndef _GTP_H_
#define _GTP_H_

#include "stdafx.h" //ここにあったらインクルード・ライブラリは "stdafx.h"に移動しました。 Youichiro Miyake //Include files are moved to "stdafx.h"

class gtp {

 public:
  enum STATUS{
    OK,
    FATAL,
    QUIT,
    SUCCESS,
    FAILURE,
  };

  gtp(engine* e, std::istream& i=std::cin, std::ostream& o=std::cout);
  virtual ~gtp();
  void  main_loop(void);

 private:
  int current_id;

  std::istream& in;
  std::ostream& out;

  float komi;
  int boardsize;
  engine* go_engine;

  typedef std::map<std::string, STATUS (gtp::*)(std::vector<std::string>&)> command_map_t;
  command_map_t gtp_command_map;
  

  STATUS gtp_success(std::string s);
  STATUS gtp_failure(std::string s);
  void gtp_panic();

  std::vector<std::string> preprocess_line(std::string& line);
  std::string  encode_coord(int i, int j);
  bool    decode_coord(std::string& s, int& i, int& j); 
  COLOR   decode_color(std::string& s);

  // GTP command
  // name
  STATUS cmd_name(std::vector<std::string>& arg);

  // version
  STATUS cmd_version(std::vector<std::string>& arg);

  // protocol_version
  STATUS cmd_protocol_version(std::vector<std::string>& arg);

  // quit
  STATUS cmd_quit(std::vector<std::string>& arg);

  // boardsize
  STATUS cmd_boardsize(std::vector<std::string>& arg);

  // komi
  STATUS cmd_komi(std::vector<std::string>& arg);

  // clear_board
  STATUS cmd_clear_board(std::vector<std::string>& arg);

  // genmove
  STATUS cmd_genmove(std::vector<std::string>& arg);

  // play
  STATUS cmd_play(std::vector<std::string>& arg);

  // undo
  STATUS cmd_undo(std::vector<std::string>& arg);

  STATUS cmd_known_command(std::vector<std::string>& arg);
  STATUS cmd_list_commands(std::vector<std::string>& arg);

  STATUS cmd_showboard(std::vector<std::string>& arg);
  STATUS cmd_final_score(std::vector<std::string>& arg);

  STATUS cmd_time_settings(std::vector<std::string>& arg);
  STATUS cmd_time_left(std::vector<std::string>& arg);
};
#endif//_GTP_H_
